#include <stdio.h>
#include <string.h>

long i;

FILE* infile;
FILE* outfile;


char ChunkID[5],Format[5],Subchunk1ID[5],Subchunk2ID[5];
char infilename[256];
char outfilename[256];
char vszfilename[256];
char stylefilename[256];

char c1,c2;

long ChunkSize,Subchunk1Size,AudioFormat;
long NumChannels,SampleRate,ByteRate,BlockAlign;
long BitsPerSample,BytesPerSample;

unsigned long Subchunk2Size;

long blocksize;

long i,ok,l_int,r_int,icut,icut2,icut24,icut224;
long li,ri,slat;

double duration,dt,dt_ms,t_ms,duration_ms,scale16,scale24;

void get4(char*);
long get4int(void);
long get3int(void);
long get2int(void);
long read_header(void);

void make_vsz_file(void);

int main(int argc,char*argv[])
{
  icut=32768; icut2=2*icut;
  icut24=icut*256; icut224=2*icut24;
  scale16=1.0/(double)icut;
  scale24=1.0/(double)icut24;
  
  printf("ReadWAVHeader v 1.01 (6 May 2010)\n");
  printf("Reads the start of basic LPCM Wave files\n");
  printf("and reports the results.\n\n");
  
  i=argc;
  printf("Number of arguments given = %ld\n\n",i);

  if(i>0) printf("Application at  = %s\n",argv[0]);
  
  sprintf(outfilename,"%s",argv[0]);
  
  slat=(long)strrchr(outfilename,'/')-(long)outfilename;
  outfilename[slat]=(char)0;
  sprintf(vszfilename,"%s/import.vsz\0",outfilename);
  sprintf(stylefilename,"%s/style.vsz\0",outfilename);
  sprintf(outfilename,"%s/waveforms.csv\0",outfilename);
  
  
  printf("outfile name %s\n",outfilename);
  
  if(i>1)
  {
    printf("Input File at = %s\n",argv[1]);

    sprintf(infilename,"%s\0",argv[1]);

    infile=fopen(argv[1],"rt");

    i=read_header();

    printf("%ld\n",i);
    
    printf("Duration = %6.2lf sec\n",(double)Subchunk2Size/(double)ByteRate);
    
    
    dt=1000.0/(double)SampleRate; t_ms=0.0;
    
    outfile=fopen(outfilename,"wt");
    
    fprintf(outfile," \"Time ms\", \"left\", \"right\",\n");
    
    printf("dt %8.4lf milliseconds\n",dt);
    printf("First few values in sequence are:\n\n");
    i=0;
    if (BytesPerSample==2)
    {
      do
      {
        li=get2int();
        if (li>icut) li-=icut2;
        if(NumChannels==2)
        {
          ri=get2int();
          if (ri>icut) ri-=icut2;
        }
        else
        {
          ri=li;
        }
        if(i<16) printf("%8.4lf,  %4ld,  %4ld\n",t_ms,li,ri);
        fprintf(outfile,"%8.4lf,  %8.6lf,  %8.6lf,\n",t_ms,scale16*(double)li,scale16*(double)ri);
        t_ms+=dt;
        i++;
      } while (t_ms<=50.0);
    }
      
    if (BytesPerSample==3)
    {
      do
      {
        li=get3int();
        if (li>icut24) li-=icut224;
        if(NumChannels==2)
        {
          ri=get3int();
          if (ri>icut24) ri-=icut224;
        }
        else
        {
          ri=li;
        }
        if (i<16) printf("%8.4lf,  %4ld,  %4ld\n",t_ms,li,ri);
        fprintf(outfile,"%8.4lf,  %8.6lf,  %8.6lf,\n",t_ms,scale24*(double)li,scale24*(double)ri);
        
        t_ms+=dt;
        i++;
      } while (t_ms<=50.0);
    }
    
    fclose(outfile);
    fclose(infile);
    
    printf("\n\nFirst 50 ms of sample values from input WAV file\n");
    printf("saved inside app directory as 'waveforms.csv'\n");
    printf("Values scaled to +/- 1.0 being max amplitude.\n\n");
    
    make_vsz_file();

  }
  else
  {
    printf("No input file!\n");
    printf("Drag a WAV file to the ReadWAVHeader application icon\n");
    printf("to see the results\n");
  }
       
  printf("Done!\n");
  return 0;
}

void make_vsz_file(void)
{
  int vszcount,vszcheck;
  char vszchar;
  printf("Making Veusz import file = %s\n",vszfilename);
  printf(" using style from = %s\n\n",stylefilename);
  
  outfile=fopen(vszfilename,"wt");
  
  fprintf(outfile,"# This file can be used to load output from\n# ReadWAVHeader into Veusz\n");
  fprintf(outfile,"# Waveforms from start of file\n#   %s\n\n",infilename);
  
  fprintf(outfile,"ImportFileCSV('%s', linked=True)\n\n",outfilename);
  
  infile=fopen(stylefilename,"rt");
  vszcount=0;
  do
  {
    vszcheck=(int)fscanf(infile,"%c",&vszchar);
    if(vszcheck==1)
    {
      fprintf(outfile,"%c",vszchar);
    }
    else
    {
      vszcount=1000000;
    }
    
    
    vszcount++;
   } while(vszcount<2000);
  
  
  fclose(outfile);
  fclose(infile);
  printf("Vesuz file saved inside app directory as 'import.vsz'\n");
}

long read_header(void)
{
  long irh;

   printf("Reading header\n");
    
    get4(ChunkID);
    printf("ChunkID = %s\n",ChunkID);
    ChunkSize=get4int();
    printf("Chunk Size = %ld\n",ChunkSize);
    get4(Format);
    printf("Format = %s\n",Format);
    get4(Subchunk1ID);
    printf("Subchunk1ID = %s\n",Subchunk1ID);
    Subchunk1Size=get4int();
    AudioFormat=get2int();
    if((Subchunk1Size==16)&&(AudioFormat==1))
    {
      printf("LPCM format OK\n");
    }
    else
    {
      printf("Format unknown!\n");
    }
    NumChannels=get2int();
    printf("Number of Channels = %ld\n",NumChannels);
    SampleRate=get4int();
    printf("Sample Rate = %ld\n",SampleRate);
    ByteRate=get4int();
    printf("Byte Rate = %ld\n",ByteRate);
    BlockAlign=get2int();
    printf("Block Align = %ld\n",BlockAlign);
    BitsPerSample=get2int();
    printf("Bits per sample = %ld\n",BitsPerSample);
    BytesPerSample=BitsPerSample/8;
    
    printf("Bytes per sample = %ld\n",BytesPerSample);
    
    get4(Subchunk2ID);
    printf("Subchunk2ID = %s\n",Subchunk2ID);
    Subchunk2Size=(unsigned long)get4int();
    printf("Subchunk2Size = %lu [Number of Data payload bytes]\n",Subchunk2Size);
    
  irh=ByteRate;
  return irh;
}

long get3int(void)
{
  long result;
  unsigned char cip1,cip2,cip3;
  
  fscanf(infile,"%c",&cip1);
  fscanf(infile,"%c",&cip2);
  fscanf(infile,"%c",&cip3);
  result=(long)cip3;
  result=(long)cip2+256*result;
  result=(long)cip1+256*result;

  return result;
} 

long get2int(void)
{
  long result;
  unsigned char cip1,cip2;
  
  fscanf(infile,"%c",&cip1);
  fscanf(infile,"%c",&cip2);

  result=(long)cip2;
  result=(long)cip1+256*result;

  return result;
} 

long get4int(void)
{
  long result;
  unsigned char cip1,cip2,cip3,cip4;
  
  fscanf(infile,"%c",&cip1);
  fscanf(infile,"%c",&cip2);
  fscanf(infile,"%c",&cip3);
  fscanf(infile,"%c",&cip4);
    
  result=(long)cip4;
  result=(long)cip3+256*result;
  result=(long)cip2+256*result;
  result=(long)cip1+256*result;
  
  return result;
} 
void get4(char* outstring)
{
  long isp;
  char csp;
  isp=0;
  do
  {
    fscanf(infile,"%c",&csp);
    outstring[isp]=csp;
    isp++;
  } while (isp<4);
  outstring[4]=(char)0;

}









